

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include <util/delay.h>


#define DDR_OFFSET (-1)
#define PIN_OFFSET (-2)

// PORT #1

#define DISP_a_PORT     PORTC
#define DISP_a_PIN      (1<<PC2)
#define DISP_b_PORT     PORTC
#define DISP_b_PIN      (1<<PC3)
#define DISP_c_PORT     PORTC
#define DISP_c_PIN      (1<<PC5)
#define DISP_d_PORT     PORTD
#define DISP_d_PIN      (1<<PD1)

#define DISP_e_PORT     PORTD
#define DISP_e_PIN      (1<<PD0)
#define DISP_f_PORT     PORTC
#define DISP_f_PIN      (1<<PC0)
#define DISP_g_PORT     PORTC
#define DISP_g_PIN      (1<<PC1)
#define DISP_P_PORT     PORTC
#define DISP_P_PIN      (1<<PC4)

#define DISP_MUX_0_PORT     PORTB
#define DISP_MUX_0_PIN      (1<<PB0)
#define DISP_MUX_1_PORT     PORTB
#define DISP_MUX_1_PIN      (1<<PB1)

#define INKR_A_PORT         PORTD
#define INKR_A_PIN          (1<<PD2)
#define INKR_B_PORT         PORTD
#define INKR_B_PIN          (1<<PD3)

#define BUZZER_PORT         PORTB
#define BUZZER_PIN          (1<<PB2)

#define BATT_STATUS_PORT    PORTD
#define BATT_STATUS_PIN     (1<<PD6)


#define SET_TO_OUTPUT(PORT,PIN) ( (*(&PORT+DDR_OFFSET)) |= PIN )
#define SET_TO_INPUT(PORT,PIN)  ( (*(&PORT+DDR_OFFSET)) &= ~PIN )
#define SET_HIGH(PORT,PIN)      ( PORT |= PIN )
#define SET_LOW(PORT,PIN)       ( PORT &= ~PIN )
#define IN(PORT,PIN)            ( ((*(&PORT+PIN_OFFSET))&PIN) != 0 )


#define INKR_FLAG_A_FIRST   0x01
#define INKR_FLAG_B_FIRST   0x02
#define INKR_FLAG_AB        0x04
#define INKR_FLAG_A_LAST    0x08
#define INKR_FLAG_B_LAST    0x10


#define FLAG_INKR_ERROR      0x01
#define FLAG_BATT_LOW        0x02
#define FLAG_BEEP            0x04
#define FLAG_POWER_DOWN      0x08
#define FLAG_DISPLAY_DIMM    0x10


#define TIMER2_INT_ENABLE()    ( TIMSK2 = (1<<OCIE2A) )
#define TIMER2_INT_DISABLE()   ( TIMSK2 = 0 )

// ExtInt // #define INKR_INT_ENABLE()      ( EIMSK = (1<<INT1) | (1<<INT0) )
// ExtInt // #define INKR_INT_DISABLE()     ( EIMSK = 0 )

#define INKR_INT_ENABLE()      ( PCMSK2 |= _BV(PCINT18) | _BV(PCINT19) )
#define INKR_INT_DISABLE()     ( PCMSK2 &= ~(_BV(PCINT18) | _BV(PCINT19)) )

#define INKR_PULL_UP_ENABLE()  { SET_HIGH(INKR_A_PORT, INKR_A_PIN); SET_HIGH(INKR_B_PORT, INKR_B_PIN); }
#define INKR_PULL_UP_DISABLE() { SET_LOW (INKR_A_PORT, INKR_A_PIN); SET_LOW (INKR_B_PORT, INKR_B_PIN); }

#define BATT_STATUS_ENABLE()     ( ACSR = _BV(ACBG) | (ACSR & ~_BV(ACD)) )
#define BATT_STATUS_DISABLE()    ( ACSR = _BV(ACD) | (ACSR & ~_BV(ACBG)) )

#define BATT_STATUS_LOW      ( (ACSR & _BV(ACO)) != 0 )

#define DISPLAY_LOW         200
#define DISPLAY_BARS        201
#define DISPLAY_DOT         100
#define DISPLAY_OFF         0x0ff

enum {
  INKR_SA1,
  INKR_SA2,
  INKR_SB1,
  INKR_SB2,
} gInkrState;

volatile int8_t gInkrCount = 0;
uint8_t gFlags = 0;
volatile uint8_t gDisplayDigit0 = 0xff, gDisplayDigit1 = 0xff;
volatile uint8_t gDimmLimit = 3;
volatile uint8_t gTimer1ms = 0;

void SystemInit(void);
void Timer2Init(void);
void DisplayInit(void);
void InkrementInit(void);
void BuzzerInit(void);
void BattStatusInit(void);


void DisplayPoll1ms(void);
void Display(uint8_t set);
void DisplaySet(uint8_t set);
void DisplaySetDigit(uint8_t set);
void DisplayDimmPoll50ms(void);

//void InkrementPoll1ms(void);

void TimerPoll50ms(void);

void BattStatusPoll50ms(void);

//void PowerDown(void);

void BuzzerSet(uint8_t on);

int main (void)
{
//  uint8_t display = 0;
  uint8_t timer = 0;
  uint8_t i = 0;

  SystemInit();
  DisplayInit();
  InkrementInit();
  BuzzerInit();
  Timer2Init();
  BattStatusInit();

  PRR = _BV(PRTWI) | _BV(PRTIM0) | _BV(PRTIM1) | _BV(PRSPI) | _BV(PRUSART0) | _BV(PRADC);
  
  sei();
    gFlags &= ~FLAG_DISPLAY_DIMM;
  while( 1 ) {
//    PowerDown();
//    if (gTimer1ms == 0) {
//      continue; 
//    }
//    cli();
//    i += gTimer1ms;
//    gTimer1ms = 0;
//    sei();
//    InkrementPoll1ms();
    if( (gFlags & FLAG_POWER_DOWN) != 0 ) {
      set_sleep_mode(SLEEP_MODE_PWR_SAVE); //       set_sleep_mode(SLEEP_MODE_PWR_DWN);
      sleep_mode();
      if (gInkrCount != 0) {
	gFlags &= ~FLAG_POWER_DOWN;
	timer = 0;
	// Turn on internal pull-up
	// Enable analog reference
	INKR_PULL_UP_ENABLE();
        BATT_STATUS_ENABLE();
      }
    }
    while (i != gTimer1ms) {

      timer++;
      if(timer == 3 && (gFlags & FLAG_POWER_DOWN) == 0) {
	BattStatusPoll50ms();
      }

      if(timer >= 10 ) {
        DisplayDimmPoll50ms();
        TimerPoll50ms();
        timer = 0;
//	Display(88);
      }

      i+= 1;
    }
  }
}


ISR(TIMER2_COMPA_vect)
{
//static uint8_t count = 0;
//  if(gTimer1ms < 127)
    gTimer1ms++;
//  count++;
//  if (count > 4) {
    DisplayPoll1ms();
//    count = 0;
//  }
}

/* ExtInt

ISR(INT0_vect) // A falling edge
{
  // INT0 and INT1 interrupts off
  // global interrupt enable
  INKR_INT_DISABLE();
  sei(); // open path for Timer2
  if( IN(INKR_B_PORT, INKR_B_PIN) ) {
    gInkrState = INKR_SA1;
  } else {
    if (gInkrState == INKR_SB1) { // prev
      gInkrCount--;
    }
    gInkrState = INKR_SB2;
  }
  EIFR = _BV(INTF0); // ignore bounce   // clear INT0 flag ?
  INKR_INT_ENABLE();
}

ISR(INT1_vect) // B falling edge
{
  INKR_INT_DISABLE();
  sei();
  if( IN(INKR_A_PORT, INKR_A_PIN) ) {
    gInkrState = INKR_SB1;
  } else {
    if (gInkrState == INKR_SA1) { // prev
      gInkrCount++;
    }
    gInkrState = INKR_SA2;
  }
  EIFR = _BV(INTF1); // ignore bounce   // clear INT0 flag ?
  INKR_INT_ENABLE();
}

ExtInt */

ISR(PCINT2_vect)
{
static int8_t sPrevA = 0;
static int8_t sPrevB = 0; // previous pin state
  INKR_INT_DISABLE();
  sei();
  if( !IN(INKR_A_PORT, INKR_A_PIN) && sPrevA != 0) { // falling edge A
    if( IN(INKR_B_PORT, INKR_B_PIN) ) {
      gInkrState = INKR_SA1;
    } else {
      if (gInkrState == INKR_SB1) { // prev
        gInkrCount--;
      }
      gInkrState = INKR_SB2;
    }
    
    sPrevA = 0;
  } else if ( !IN(INKR_B_PORT, INKR_B_PIN) && sPrevB != 0) { // falling edge B
    if( IN(INKR_A_PORT, INKR_A_PIN) ) {
      gInkrState = INKR_SB1;
    } else {
      if (gInkrState == INKR_SA1) { // prev
        gInkrCount++;
      }
      gInkrState = INKR_SA2;
    }

    sPrevB = 0;
  } else { // avoid lock out
    sPrevA = 1;
    sPrevB = 1;
  }
  

  PCIFR = _BV(PCIF2); // ignore bounce
  INKR_INT_ENABLE();  
}


void SystemInit(void)
{
  ACSR = (1<<ACD);

  PORTB = 0x0ff;
  PORTC = 0x0ff ;
  PORTD = 0x0ff;

  DDRB = 0x0;
  DDRC = 0x0;
  DDRD = 0x0;
}

void BuzzerInit(void)
{
  SET_TO_OUTPUT(BUZZER_PORT, BUZZER_PIN);
  SET_LOW      (BUZZER_PORT, BUZZER_PIN);
}

void Timer2Init(void)
{
// Interrupt enable
// Clock select
  TCCR2A = (1<<WGM21);
  TCCR2B = (1<<CS21);
//TCCR2B = (1<<CS20);
  TCNT2  = 0;
  OCR2A = 70; //16; // 1ms 128 kHz
//  OCR2A  = 125; // 1ms 8MHz
// OCR2A = 128;
//  OCR2B
//  TIMSK2 = (1<<OCIE2A);
  TIMER2_INT_ENABLE();
}


void DisplayInit(void)
{
  SET_TO_OUTPUT(DISP_MUX_0_PORT, DISP_MUX_0_PIN);
  SET_LOW      (DISP_MUX_0_PORT, DISP_MUX_0_PIN);

  SET_TO_OUTPUT(DISP_MUX_1_PORT, DISP_MUX_1_PIN);
  SET_LOW      (DISP_MUX_1_PORT, DISP_MUX_1_PIN);

  SET_TO_OUTPUT(DISP_a_PORT, DISP_a_PIN);
  SET_LOW      (DISP_a_PORT, DISP_a_PIN);
  SET_TO_OUTPUT(DISP_b_PORT, DISP_b_PIN);
  SET_LOW      (DISP_b_PORT, DISP_b_PIN);
  SET_TO_OUTPUT(DISP_c_PORT, DISP_c_PIN);
  SET_LOW      (DISP_c_PORT, DISP_c_PIN);
  SET_TO_OUTPUT(DISP_d_PORT, DISP_d_PIN);
  SET_LOW      (DISP_d_PORT, DISP_d_PIN);

  SET_TO_OUTPUT(DISP_e_PORT, DISP_e_PIN);
  SET_LOW      (DISP_e_PORT, DISP_e_PIN);
  SET_TO_OUTPUT(DISP_f_PORT, DISP_f_PIN);
  SET_LOW      (DISP_f_PORT, DISP_f_PIN);
  SET_TO_OUTPUT(DISP_g_PORT, DISP_g_PIN);
  SET_LOW      (DISP_g_PORT, DISP_g_PIN);
  SET_TO_OUTPUT(DISP_P_PORT, DISP_P_PIN);
  SET_LOW      (DISP_P_PORT, DISP_P_PIN);
 
}


void InkrementInit(void)
{
  SET_TO_INPUT(INKR_A_PORT, INKR_A_PIN);
  SET_HIGH    (INKR_A_PORT, INKR_A_PIN);
  SET_TO_INPUT(INKR_B_PORT, INKR_B_PIN);
  SET_HIGH    (INKR_B_PORT, INKR_B_PIN);
  
  
  // TODO PCI bitmask
  // ExtInt // EICRA = _BV(ISC01) | _BV(ISC11);
  PCICR = _BV(PCIE2); // Enable interrupt controler
  INKR_INT_ENABLE(); // Enable interrput
//  EIMSK = _BV(INT0) | _BV(INT1);
}

void BattStatusInit(void)
{
  SET_TO_INPUT(BATT_STATUS_PORT, BATT_STATUS_PIN);
  SET_LOW     (BATT_STATUS_PORT, BATT_STATUS_PIN);
  ADCSRB = 0;
  DIDR1 = _BV(AIN1D);  
}


void Display(uint8_t set)
{
  uint8_t a, b;
  uint8_t temp = set;
  if(set == 0xff) {
    gDisplayDigit0 = 0xff;
    gDisplayDigit1 = 0xff;
  } else {
    if(set < 100) {
      a = temp / 10;
      b = set % 10;
    } else if(set < 200) {
      temp -= 100;
      a = temp / 10;
      b = temp % 10;
      b += 10;
    } else if(set == 200) {
      a = 21;
      b = 22;
    } else {
      a = 20;
      b = 20;
    }
    cli();
    gDisplayDigit0 = b;
    gDisplayDigit1 = a;
    sei();
  }  
}

void DisplayPoll1ms(void)
{
static uint8_t sCount = 0;
  
  if(gDisplayDigit0 == 0xff) {
      SET_LOW(DISP_MUX_0_PORT, DISP_MUX_0_PIN);
      SET_LOW(DISP_MUX_1_PORT, DISP_MUX_1_PIN);
      SET_LOW (DISP_a_PORT, DISP_a_PIN);
      SET_LOW (DISP_b_PORT, DISP_b_PIN);
      SET_LOW (DISP_c_PORT, DISP_c_PIN);
      SET_LOW (DISP_d_PORT, DISP_d_PIN);
      SET_LOW (DISP_e_PORT, DISP_e_PIN);
      SET_LOW (DISP_f_PORT, DISP_f_PIN);
      SET_LOW (DISP_g_PORT, DISP_g_PIN);
  } else {
    if ( sCount == 0 ) { // || (sCount == 1 && (gFlags & FLAG_DISPLAY_DIMM) == 0) ) {
//      DisplaySet(gDisplayDigit1|0x80);
      SET_LOW(DISP_MUX_1_PORT, DISP_MUX_1_PIN);
//      SET_LOW(DISP_MUX_1_PORT, DISP_MUX_1_PIN);
      DisplaySetDigit( gDisplayDigit0);
      SET_HIGH(DISP_MUX_0_PORT, DISP_MUX_0_PIN);

    } else if ( sCount == 1 ) { // || (sCount == 3 && (gFlags & FLAG_DISPLAY_DIMM) == 0) ) {
      SET_LOW(DISP_MUX_0_PORT, DISP_MUX_0_PIN);
//      SET_LOW(DISP_MUX_1_PORT, DISP_MUX_1_PIN);
      DisplaySetDigit(gDisplayDigit1);
      SET_HIGH(DISP_MUX_1_PORT, DISP_MUX_1_PIN);
    } else {
      SET_LOW(DISP_MUX_1_PORT, DISP_MUX_1_PIN);
      SET_LOW(DISP_MUX_0_PORT, DISP_MUX_0_PIN);
    }
  }
  sCount++;
  if(sCount >= gDimmLimit ) {
    sCount = 0;
  }
}


void DisplaySet(uint8_t set)
{
// set
//  Bit6-7 select
//  Bit0-4 digits 0-9 10-19 20 -> - 21 -> l 22 -> o 0x1f -> off
//  uint8_t
  SET_LOW(DISP_MUX_0_PORT, DISP_MUX_0_PIN);
  SET_LOW(DISP_MUX_1_PORT, DISP_MUX_1_PIN);

  DisplaySetDigit( set);
  
  if ((set&0x40) != 0) {
    SET_HIGH(DISP_MUX_0_PORT, DISP_MUX_0_PIN);
    SET_LOW (DISP_MUX_1_PORT, DISP_MUX_1_PIN);
  } else if( (set&0x80) != 0 ) {
    SET_LOW (DISP_MUX_0_PORT, DISP_MUX_0_PIN);
    SET_HIGH(DISP_MUX_1_PORT, DISP_MUX_1_PIN);
  } else { // off
  }
}

void DisplaySetDigit(uint8_t set)
{
 // 0..9 => 0..9
 // 10..19 => 0..9 mit Punkt
 // 20 => -
 // 21 => L
 // 22 => o
 // 23 => off
  set &= 0x1f;
  if ( set < 20 ) {
    if( set > 9 ) { // dot
      SET_HIGH(DISP_P_PORT, DISP_P_PIN);
      set -= 10;
    } else { // no dot
      SET_LOW (DISP_P_PORT, DISP_P_PIN);
    }
    if ( set == 1 ) {
      SET_LOW (DISP_a_PORT, DISP_a_PIN);
      SET_HIGH(DISP_b_PORT, DISP_b_PIN);
      SET_HIGH(DISP_c_PORT, DISP_c_PIN);
      SET_LOW (DISP_d_PORT, DISP_d_PIN);
      SET_LOW (DISP_e_PORT, DISP_e_PIN);
      SET_LOW (DISP_f_PORT, DISP_f_PIN);
      SET_LOW (DISP_g_PORT, DISP_g_PIN);
    } else if ( set == 2 ) {
      SET_HIGH(DISP_a_PORT, DISP_a_PIN);
      SET_HIGH(DISP_b_PORT, DISP_b_PIN);
      SET_LOW (DISP_c_PORT, DISP_c_PIN);
      SET_HIGH(DISP_d_PORT, DISP_d_PIN);
      SET_HIGH(DISP_e_PORT, DISP_e_PIN);
      SET_LOW (DISP_f_PORT, DISP_f_PIN);
      SET_HIGH(DISP_g_PORT, DISP_g_PIN);
   } else if ( set == 3 ) {
      SET_HIGH(DISP_a_PORT, DISP_a_PIN);
      SET_HIGH(DISP_b_PORT, DISP_b_PIN);
      SET_HIGH(DISP_c_PORT, DISP_c_PIN);
      SET_HIGH(DISP_d_PORT, DISP_d_PIN);
      SET_LOW (DISP_e_PORT, DISP_e_PIN);
      SET_LOW (DISP_f_PORT, DISP_f_PIN);
      SET_HIGH(DISP_g_PORT, DISP_g_PIN);
   } else if ( set == 4 ) {
      SET_LOW (DISP_a_PORT, DISP_a_PIN);
      SET_HIGH(DISP_b_PORT, DISP_b_PIN);
      SET_HIGH(DISP_c_PORT, DISP_c_PIN);
      SET_LOW (DISP_d_PORT, DISP_d_PIN);
      SET_LOW (DISP_e_PORT, DISP_e_PIN);
      SET_HIGH(DISP_f_PORT, DISP_f_PIN);
      SET_HIGH(DISP_g_PORT, DISP_g_PIN);
   } else if ( set == 5 ) {
      SET_HIGH(DISP_a_PORT, DISP_a_PIN);
      SET_LOW (DISP_b_PORT, DISP_b_PIN);
      SET_HIGH(DISP_c_PORT, DISP_c_PIN);
      SET_HIGH(DISP_d_PORT, DISP_d_PIN);
      SET_LOW(DISP_e_PORT, DISP_e_PIN);
      SET_HIGH(DISP_f_PORT, DISP_f_PIN);
      SET_HIGH(DISP_g_PORT, DISP_g_PIN);
   } else if ( set == 6 ) {
      SET_HIGH(DISP_a_PORT, DISP_a_PIN);
      SET_LOW (DISP_b_PORT, DISP_b_PIN);
      SET_HIGH(DISP_c_PORT, DISP_c_PIN);
      SET_HIGH(DISP_d_PORT, DISP_d_PIN);
      SET_HIGH(DISP_e_PORT, DISP_e_PIN);
      SET_HIGH(DISP_f_PORT, DISP_f_PIN);
      SET_HIGH(DISP_g_PORT, DISP_g_PIN);
   } else if ( set == 7 ) {
      SET_HIGH(DISP_a_PORT, DISP_a_PIN);
      SET_HIGH(DISP_b_PORT, DISP_b_PIN);
      SET_HIGH(DISP_c_PORT, DISP_c_PIN);
      SET_LOW (DISP_d_PORT, DISP_d_PIN);
      SET_LOW (DISP_e_PORT, DISP_e_PIN);
      SET_LOW (DISP_f_PORT, DISP_f_PIN);
      SET_LOW (DISP_g_PORT, DISP_g_PIN);
   } else if (set == 8) {
      SET_HIGH(DISP_a_PORT, DISP_a_PIN);
      SET_HIGH(DISP_b_PORT, DISP_b_PIN);
      SET_HIGH(DISP_c_PORT, DISP_c_PIN);
      SET_HIGH(DISP_d_PORT, DISP_d_PIN);
      SET_HIGH(DISP_e_PORT, DISP_e_PIN);
      SET_HIGH(DISP_f_PORT, DISP_f_PIN);
      SET_HIGH(DISP_g_PORT, DISP_g_PIN);
   } else if (set == 9) {
      SET_HIGH(DISP_a_PORT, DISP_a_PIN);
      SET_HIGH(DISP_b_PORT, DISP_b_PIN);
      SET_HIGH(DISP_c_PORT, DISP_c_PIN);
      SET_HIGH(DISP_d_PORT, DISP_d_PIN);
      SET_LOW (DISP_e_PORT, DISP_e_PIN);
      SET_HIGH(DISP_f_PORT, DISP_f_PIN);
      SET_HIGH(DISP_g_PORT, DISP_g_PIN);
   } else if (set == 0) {
      SET_HIGH(DISP_a_PORT, DISP_a_PIN);
      SET_HIGH(DISP_b_PORT, DISP_b_PIN);
      SET_HIGH(DISP_c_PORT, DISP_c_PIN);
      SET_HIGH(DISP_d_PORT, DISP_d_PIN);
      SET_HIGH(DISP_e_PORT, DISP_e_PIN);
      SET_HIGH(DISP_f_PORT, DISP_f_PIN);
      SET_LOW(DISP_g_PORT, DISP_g_PIN);
    }
  } else if ( set == 20 ) { // -
    SET_LOW (DISP_a_PORT, DISP_a_PIN);
    SET_LOW (DISP_b_PORT, DISP_b_PIN);
    SET_LOW (DISP_c_PORT, DISP_c_PIN);
    SET_LOW (DISP_d_PORT, DISP_d_PIN);
    SET_LOW (DISP_e_PORT, DISP_e_PIN);
    SET_LOW (DISP_f_PORT, DISP_f_PIN);
    SET_HIGH(DISP_g_PORT, DISP_g_PIN);
    SET_LOW (DISP_P_PORT, DISP_P_PIN);

  } else if ( set == 21 ) { // L
    SET_LOW (DISP_a_PORT, DISP_a_PIN);
    SET_LOW (DISP_b_PORT, DISP_b_PIN);
    SET_LOW (DISP_c_PORT, DISP_c_PIN);
    SET_HIGH(DISP_d_PORT, DISP_d_PIN);
    SET_HIGH(DISP_e_PORT, DISP_e_PIN);
    SET_HIGH(DISP_f_PORT, DISP_f_PIN);
    SET_LOW (DISP_g_PORT, DISP_g_PIN);
    SET_LOW (DISP_P_PORT, DISP_P_PIN);
  } else if (set == 22 ) { // o
    SET_LOW (DISP_a_PORT, DISP_a_PIN);
    SET_LOW (DISP_b_PORT, DISP_b_PIN);
    SET_HIGH(DISP_c_PORT, DISP_c_PIN);
    SET_HIGH(DISP_d_PORT, DISP_d_PIN);
    SET_HIGH(DISP_e_PORT, DISP_e_PIN);
    SET_LOW (DISP_f_PORT, DISP_f_PIN);
    SET_HIGH(DISP_g_PORT, DISP_g_PIN);
    SET_LOW (DISP_P_PORT, DISP_P_PIN);
  } else { // off
    SET_LOW (DISP_a_PORT, DISP_a_PIN);
    SET_LOW (DISP_b_PORT, DISP_b_PIN);
    SET_LOW (DISP_c_PORT, DISP_c_PIN);
    SET_LOW (DISP_d_PORT, DISP_d_PIN);
    SET_LOW (DISP_e_PORT, DISP_e_PIN);
    SET_LOW (DISP_f_PORT, DISP_f_PIN);
    SET_HIGH(DISP_g_PORT, DISP_g_PIN);
    SET_LOW (DISP_P_PORT, DISP_P_PIN);
  }  
}


void DisplayDimmPoll50ms(void)
{
static uint16_t sDimmTimer50ms = 0;
  if(gInkrCount != 0) {
    sDimmTimer50ms = 60;
  }
  if ( sDimmTimer50ms != 0) {
    gDimmLimit = 2;
    sDimmTimer50ms--;
  } else {
    gDimmLimit = 3;
  }
  
  
}



void InkrementPoll1ms(void)
{
static uint8_t sFlags = 0;
// static uint16_t sTimeout = 0;

  if ( sFlags == 0 && IN(INKR_A_PORT, INKR_A_PIN) && IN(INKR_B_PORT, INKR_B_PIN) ) {
    goto
      finish;
  }

  if ( sFlags == 0 ) {
    if( !IN(INKR_A_PORT, INKR_A_PIN) && IN(INKR_B_PORT, INKR_B_PIN) ) {
      sFlags = INKR_FLAG_A_FIRST;
    } else if (IN(INKR_A_PORT, INKR_A_PIN) && !IN(INKR_B_PORT, INKR_B_PIN)) {
      sFlags = INKR_FLAG_B_FIRST;

    }
    goto
      finish;
  }

  if ( !IN(INKR_A_PORT, INKR_A_PIN) && !IN(INKR_B_PORT, INKR_B_PIN) ) {
    sFlags |= INKR_FLAG_AB;
    goto
      finish;
  }

  if ( (sFlags&INKR_FLAG_AB) != 0 ) {
    if( !IN(INKR_A_PORT, INKR_A_PIN) && IN(INKR_B_PORT, INKR_B_PIN) ) {
      sFlags = (sFlags & ~INKR_FLAG_B_LAST) | INKR_FLAG_A_LAST;
      goto
        finish;
    } else if (IN(INKR_A_PORT, INKR_A_PIN) && !IN(INKR_B_PORT, INKR_B_PIN)) {
      sFlags = (sFlags & ~INKR_FLAG_A_LAST) | INKR_FLAG_B_LAST;
      goto
        finish;
    }
  }

  if ( (sFlags == (INKR_FLAG_A_FIRST | INKR_FLAG_AB | INKR_FLAG_B_LAST)) &&
      (IN(INKR_A_PORT, INKR_A_PIN) && IN(INKR_B_PORT, INKR_B_PIN)) ) {
    if(gInkrCount < 20) {
      gInkrCount++;
    }
    goto
      finish;
  }

  if ( (sFlags == (INKR_FLAG_B_FIRST | INKR_FLAG_AB | INKR_FLAG_A_LAST)) &&
      (IN(INKR_A_PORT, INKR_A_PIN) && IN(INKR_B_PORT, INKR_B_PIN)) ) {
    if(gInkrCount > -20) {
      gInkrCount--;
    }
  }

finish:
  if ( IN(INKR_A_PORT, INKR_A_PIN) && IN(INKR_B_PORT, INKR_B_PIN) ) {
    sFlags = 0;
  }
/* no INKR_ERROR
    sTimeout = 0;
  } else {
    if(sTimeout < 3000)
      sTimeout++;
  }
  if ( sTimeout >= 3000 ) {
    gFlags |=  FLAG_INKR_ERROR;
  } else {
    gFlags &= ~FLAG_INKR_ERROR;
  }
*/
}



void TimerPoll50ms(void)
{
static uint8_t sTimer50ms = 0; // 0..19 
static uint8_t sTimerSeconds = 0;
static int16_t sTimerMinutes = -1;
  uint8_t temp = 0;
  int8_t inkrCount;
  if( (gFlags & FLAG_POWER_DOWN) != 0 ) {
    return;
  }


  if (sTimerMinutes == -1 && sTimerSeconds == 0 && sTimer50ms == 0) {
    gFlags |= FLAG_POWER_DOWN;
    BATT_STATUS_DISABLE();
    INKR_PULL_UP_DISABLE();
    sTimerSeconds = 60;
    sTimer50ms = 20;
    return;
  }

// Count down 
  if (sTimer50ms == 0) {
    sTimer50ms = 19;
    if(sTimerSeconds == 0) { 
      sTimerSeconds = 59;
      sTimerMinutes--;
    } else {
      sTimerSeconds--;
    }
  } else {
    sTimer50ms--;
  }
  cli();
  inkrCount = gInkrCount;
  gInkrCount = 0;
  sei();
  if (inkrCount != 0) {
    if(sTimerMinutes <= -1) { //  && sTimer100ms == 0)
       sTimerMinutes = 0;
    }
    sTimerMinutes += inkrCount;
    if(sTimerMinutes > 20) {
      sTimerMinutes += inkrCount;
      sTimerMinutes &= 0xfffe;
    }
    if(sTimerMinutes < -1) {
      sTimerMinutes = -1;
    } else {
      sTimerSeconds = 59;
      sTimer50ms = 19;
    }
    if(sTimerMinutes > 99)
      sTimerMinutes = 99;
  }

  if (sTimerMinutes >= 0) {
    temp = sTimerMinutes;
    if( sTimer50ms < 4 && sTimerMinutes != 0) {
      temp += DISPLAY_DOT;
/* no INKR_ERROR
      if ((gFlags & FLAG_INKR_ERROR) != 0) {
        temp = DISPLAY_BARS;
        gFlags |= FLAG_BEEP;
      } else
*/
      if ( (gFlags & FLAG_BATT_LOW) != 0 ) {
        temp = DISPLAY_LOW;
        gFlags |= FLAG_BEEP;
      }
      
      
    }
    if (sTimerMinutes == 0) {
      if ( sTimer50ms == 19 || sTimer50ms == 17 || sTimer50ms == 15) {
	temp = DISPLAY_OFF;
        BuzzerSet(1);
      } else {
	BuzzerSet(0);
      }
    } else {
      BuzzerSet(0);
    }

  } else {
    BuzzerSet(0);
    temp = DISPLAY_OFF;    
/* no INKR_ERROR
    if( sTimer50ms < 4) {
      if ((gFlags & FLAG_INKR_ERROR) != 0) {
        temp = DISPLAY_BARS;
        gFlags |= FLAG_BEEP;
        BuzzerSet(1);
      }

    }
*/
  }
  Display(temp);

/*
  if(sTimer100ms != 0)
    sTimer100ms--;
  if(sTimer100ms == 0 && sTimerMinutes > -1) {
//    sTimer100ms = 600;
    sTimer100ms = 60;
    sTimerMinutes--;
  }
  if(sTimer100ms == 0 && ((gFlags & FLAG_INKR_ERROR) != 0) ) {
    sTimer100ms = 60;
  }

  if(gInkCount != 0) {
    if(sTimerMinutes <= -1) { //  && sTimer100ms == 0)
       sTimerMinutes = 0;
    }
    sTimerMinutes += gInkCount;
    gInkCount = 0;
//    sTimer100ms = 600;
    sTimer100ms = 60;
    if(sTimerMinutes < -1) {
      sTimerMinutes = -1;
//      sTimer100ms = 60;
    }
    if(sTimerMinutes > 99)
      sTimerMinutes = 99;
  }

  temp = sTimerMinutes;

  if ( sTimerMinutes == -1) {
    temp = 0xff;
  }

  if(sTimer100ms%10 == 0 || sTimer100ms%10 == 9) {
    if(sTimerMinutes > 0) {
      temp += 100;
    }
    if(sTimerMinutes == 0) {
      temp = 0xff;
      gFlags |= FLAG_BEEP;
    }
    if ((gFlags & FLAG_INKR_ERROR) != 0) {
      temp = 201;
      gFlags |= FLAG_BEEP;
    } else if ( (gFlags & FLAG_BATT_LOW) != 0 ) {
      temp = 200;
      gFlags |= FLAG_BEEP;
    }

  }

//  if( (gFlags & (FLAG_INKR_ERROR | FLAG_BATT_LOW)) == 0 && sTimerMinutes != 0) {
//    gFlags &= ~FLAG_BEEP;
//  }


  if ( sTimerMinutes == -1 && sTimer100ms == 0) {
    temp = 0xff;7
    gFlags |= FLAG_POWER_DOWN;
    sTimer100ms = 60;
//    sTimerMinutes = 5;
  }

  gDisplay = temp;
  */
}

/*
void PowerDown(void)
{
  if( (gFlags & FLAG_POWER_DOWN) != 0 && IN(INKR_A_PORT, INKR_A_PIN) && IN(INKR_B_PORT, INKR_B_PIN) && (gFlags & FLAG_INKR_ERROR) == 0 ) {
// Flag löschen
// Display aus
// Schlafmodus wechseln
// Timer ausschalten
// ext Int einschalten
// Schlafen gehen
// Timer einschalten
// Schlafmodus wechseln
    DisplaySet(0);
    gFlags &= ~FLAG_POWER_DOWN;
    set_sleep_mode(SLEEP_MODE_PWR_DOWN);
    TIMER2_INT_DISABLE();
    INKR_INT_ENABLE();
    sleep_mode();
    INKR_INT_DISABLE();
    TIMER2_INT_ENABLE();
    set_sleep_mode(SLEEP_MODE_IDLE);
  }
}
*/

void BattStatusPoll50ms(void)
{
static uint8_t sCount = 0;
  if(BATT_STATUS_LOW) {
    sCount++;
    if(sCount > 20) {
      sCount = 20;
      gFlags |= FLAG_BATT_LOW;
    }
  } else {
    gFlags &= FLAG_BATT_LOW;
    sCount = 0;
  }

}

void BuzzerSet(uint8_t on)
{
  if(on != 0) {
    SET_HIGH(BUZZER_PORT, BUZZER_PIN);    
  } else {
    SET_LOW(BUZZER_PORT, BUZZER_PIN);
  }
}

// EOF
